// SNF4CGP/OutputProcessor.cpp
// Copyright (C) 2009 ARM Research Labs, LLC.
// See www.armresearch.com for more information.

#include "OutputProcessor.hpp"
#include "JobPool.hpp"

#include "../CodeDweller/faults.hpp"

using namespace std;

const ThreadType OutputProcessor::Type("Output Processor");
const ThreadState OutputProcessor::WaitingForJobs("Waiting");
const ThreadState OutputProcessor::PostingResponses("Posting");

OutputProcessor::OutputProcessor() :                                            // Constructor sets up the basics.
  Thread(OutputProcessor::Type, "Output"),                                      // Name the thread.
  RecycledJobs_(0),                                                             // No job pool yet.
  isTimeToStop(false) {                                                         // Not time to stop.
}

OutputProcessor::~OutputProcessor() {                                           // Destructor to clean things up.
    try { stop(); } catch (...) {}                                              // Call stop() just in case.
}

RuntimeCheck CheckRecyclingNotNull("OutputProcessor::RecycledJobs() Check(0 != RecycledJobs_)");

JobPool& OutputProcessor::RecycledJobs() {                                      // Safe access to RecycledJobs
    CheckRecyclingNotNull(0 != RecycledJobs_);                                  // Throw if we have a null pointer.
    return *RecycledJobs_;                                                      // Good pointer? return a reference.
}

void OutputProcessor::init(JobPool& J) {                                        // Start processing.
    RecycledJobs_ = &J;
    run();
}

void OutputProcessor::outputJob(Job& J) {                                       // Take this job and ... you know.
    CompletedJobs.give(&J);
}

void OutputProcessor::sendString(const string& S) {                             // Safely send a string to cout.
    ScopeMutex OneFlushAtATime(ProtectCout);
    cout << S;
    cout.flush();
}

void OutputProcessor::stop() {                                                  // Finish off the queue and quit.
    if(false == isTimeToStop) {                                                 // Do this only once.
        isTimeToStop = true;                                                    // Set the stop flag.
        CompletedJobs.give(0);                                                  // Add the last job to kill the loop.
    }
}

void OutputProcessor::handleJob(Job& J) {                                       // Process a job.
    CurrentThreadState(PostingResponses);
    sendString(J.Responses());
    J.clear();
    RecycledJobs().drop(J);
}

void OutputProcessor::myTask() {                                                // This is how we do it.
    Job* J = 0;                                                                 // Pull Job pointers from the queue.
    CurrentThreadState(WaitingForJobs);
    while(J = CompletedJobs.take()) {                                           // Process every job until we get
        handleJob(*J);                                                          // a 0 pointer. Then we're done.
        CurrentThreadState(WaitingForJobs);
    }
}
